Class: Datadog::Statsd
- Inherits:
-
Object
- Object
- Datadog::Statsd
- Defined in:
- lib/datadog/statsd.rb,
lib/datadog/statsd/timer.rb,
lib/datadog/statsd/sender.rb,
lib/datadog/statsd/version.rb,
lib/datadog/statsd/forwarder.rb,
lib/datadog/statsd/telemetry.rb,
lib/datadog/statsd/connection.rb,
lib/datadog/statsd/serialization.rb,
lib/datadog/statsd/connection_cfg.rb,
lib/datadog/statsd/message_buffer.rb,
lib/datadog/statsd/udp_connection.rb,
lib/datadog/statsd/uds_connection.rb,
lib/datadog/statsd/single_thread_sender.rb,
lib/datadog/statsd/serialization/serializer.rb,
lib/datadog/statsd/serialization/tag_serializer.rb,
lib/datadog/statsd/serialization/stat_serializer.rb,
lib/datadog/statsd/serialization/event_serializer.rb,
lib/datadog/statsd/serialization/service_check_serializer.rb
Defined Under Namespace
Modules: Serialization Classes: Connection, ConnectionCfg, Error, Forwarder, MessageBuffer, Sender, SingleThreadSender, Telemetry, Timer, UDPConnection, UDSConnection
Constant Summary collapse
- OK =
0- WARNING =
1- CRITICAL =
2- UNKNOWN =
3- UDP_DEFAULT_BUFFER_SIZE =
1_432- UDS_DEFAULT_BUFFER_SIZE =
8_192- DEFAULT_BUFFER_POOL_SIZE =
Float::INFINITY
- UDP_DEFAULT_SENDER_QUEUE_SIZE =
2048- UDS_DEFAULT_SENDER_QUEUE_SIZE =
512- MAX_EVENT_SIZE =
8 * 1_024
- DEFAULT_TELEMETRY_FLUSH_INTERVAL =
minimum flush interval for the telemetry in seconds
10- COUNTER_TYPE =
'c'- GAUGE_TYPE =
'g'- HISTOGRAM_TYPE =
'h'- DISTRIBUTION_TYPE =
'd'- TIMING_TYPE =
'ms'- SET_TYPE =
's'- VERSION =
'5.3.3'
Instance Attribute Summary collapse
-
#namespace ⇒ Object
readonly
A namespace to prepend to all statsd calls.
-
#sample_rate ⇒ Object
readonly
Default sample rate.
Class Method Summary collapse
-
.open(*args) ⇒ Object
yield a new instance to a block and close it when done for short-term use-cases that don’t want to close the socket manually.
Instance Method Summary collapse
-
#batch {|_self| ... } ⇒ Object
Send several metrics in the same packet.
-
#close(flush: true) ⇒ Object
Close the underlying socket.
-
#count(stat, count, opts = EMPTY_OPTIONS) ⇒ Object
Sends an arbitrary count for the given stat to the statsd server.
-
#decrement(stat, opts = EMPTY_OPTIONS) ⇒ Object
Sends a decrement (count = -1) for the given stat to the statsd server.
-
#distribution(stat, value, opts = EMPTY_OPTIONS) ⇒ Object
Sends a value to be tracked as a distribution to the statsd server.
-
#event(title, text, opts = EMPTY_OPTIONS) ⇒ Object
This end point allows you to post events to the stream.
-
#flush(flush_telemetry: false, sync: false) ⇒ Object
Flush the buffer into the connection.
-
#gauge(stat, value, opts = EMPTY_OPTIONS) ⇒ Object
Sends an arbitary gauge value for the given stat to the statsd server.
-
#histogram(stat, value, opts = EMPTY_OPTIONS) ⇒ Object
Sends a value to be tracked as a histogram to the statsd server.
- #host ⇒ Object
-
#increment(stat, opts = EMPTY_OPTIONS) ⇒ Object
Sends an increment (count = 1) for the given stat to the statsd server.
-
#initialize(host = nil, port = nil, socket_path: nil, namespace: nil, tags: nil, sample_rate: nil, buffer_max_payload_size: nil, buffer_max_pool_size: nil, buffer_overflowing_stategy: :drop, buffer_flush_interval: nil, sender_queue_size: nil, logger: nil, single_thread: false, telemetry_enable: true, telemetry_flush_interval: DEFAULT_TELEMETRY_FLUSH_INTERVAL) ⇒ Statsd
constructor
A new instance of Statsd.
- #port ⇒ Object
- #service_check(name, status, opts = EMPTY_OPTIONS) ⇒ Object
-
#set(stat, value, opts = EMPTY_OPTIONS) ⇒ Object
Sends a value to be tracked as a set to the statsd server.
- #socket_path ⇒ Object
- #sync_with_outbound_io ⇒ Object
-
#tags ⇒ Object
Global tags to be added to every statsd call.
- #telemetry ⇒ Object
-
#time(stat, opts = EMPTY_OPTIONS) { ... } ⇒ Object
Reports execution time of the provided block using #timing.
-
#timing(stat, ms, opts = EMPTY_OPTIONS) ⇒ Object
Sends a timing (in ms) for the given stat to the statsd server.
- #transport_type ⇒ Object
Constructor Details
#initialize(host = nil, port = nil, socket_path: nil, namespace: nil, tags: nil, sample_rate: nil, buffer_max_payload_size: nil, buffer_max_pool_size: nil, buffer_overflowing_stategy: :drop, buffer_flush_interval: nil, sender_queue_size: nil, logger: nil, single_thread: false, telemetry_enable: true, telemetry_flush_interval: DEFAULT_TELEMETRY_FLUSH_INTERVAL) ⇒ Statsd
Returns a new instance of Statsd.
87 88 89 90 91 92 93 94 95 96 97 98 99 100 101 102 103 104 105 106 107 108 109 110 111 112 113 114 115 116 117 118 119 120 121 122 123 124 125 126 127 128 129 130 131 132 133 134 135 136 137 138 139 140 141 142 143 144 145 146 147 148 149 150 151 152 153 |
# File 'lib/datadog/statsd.rb', line 87 def initialize( host = nil, port = nil, socket_path: nil, namespace: nil, tags: nil, sample_rate: nil, buffer_max_payload_size: nil, buffer_max_pool_size: nil, buffer_overflowing_stategy: :drop, buffer_flush_interval: nil, sender_queue_size: nil, logger: nil, single_thread: false, telemetry_enable: true, telemetry_flush_interval: DEFAULT_TELEMETRY_FLUSH_INTERVAL ) unless .nil? || .is_a?(Array) || .is_a?(Hash) raise ArgumentError, 'tags must be an array of string tags or a Hash' end @namespace = namespace @prefix = @namespace ? "#{@namespace}.".freeze : nil @serializer = Serialization::Serializer.new(prefix: @prefix, global_tags: ) @sample_rate = sample_rate # deprecation message for ruby < 2.1.0 users as we will drop support for ruby 2.0 # in dogstatsd-ruby 5.4.0 # TODO(remy): remove this message and the two global vars used in dogstatd-ruby 5.4.0 if RUBY_VERSION < '2.1.0' && .try_lock && ! if logger != nil logger.warn { "deprecation: dogstatsd-ruby will drop support of Ruby < 2.1.0 in a next minor release" } else puts("warning: deprecation: dogstatsd-ruby will drop support of Ruby < 2.1.0 in a next minor release") end = true .unlock end @forwarder = Forwarder.new( connection_cfg: ConnectionCfg.new( host: host, port: port, socket_path: socket_path, ), global_tags: , logger: logger, single_thread: single_thread, buffer_max_payload_size: buffer_max_payload_size, buffer_max_pool_size: buffer_max_pool_size, buffer_overflowing_stategy: buffer_overflowing_stategy, buffer_flush_interval: buffer_flush_interval, sender_queue_size: sender_queue_size, telemetry_flush_interval: telemetry_enable ? telemetry_flush_interval : nil, ) end |
Instance Attribute Details
#namespace ⇒ Object (readonly)
A namespace to prepend to all statsd calls. Defaults to no namespace.
65 66 67 |
# File 'lib/datadog/statsd.rb', line 65 def namespace @namespace end |
#sample_rate ⇒ Object (readonly)
Default sample rate
73 74 75 |
# File 'lib/datadog/statsd.rb', line 73 def sample_rate @sample_rate end |
Class Method Details
.open(*args) ⇒ Object
yield a new instance to a block and close it when done for short-term use-cases that don’t want to close the socket manually
157 158 159 160 161 162 163 |
# File 'lib/datadog/statsd.rb', line 157 def self.open(*args) instance = new(*args) yield instance ensure instance.close end |
Instance Method Details
#batch {|_self| ... } ⇒ Object
Send several metrics in the same packet. They will be buffered and flushed when the block finishes.
This method exists for compatibility with v4.x versions, it is not needed anymore since the batching is now automatically done internally. It also means that an automatic flush could occur if the buffer is filled during the execution of the batch block.
This method is DEPRECATED and will be removed in future v6.x API.
356 357 358 359 |
# File 'lib/datadog/statsd.rb', line 356 def batch yield self flush(sync: true) end |
#close(flush: true) ⇒ Object
Close the underlying socket
364 365 366 367 |
# File 'lib/datadog/statsd.rb', line 364 def close(flush: true) flush(sync: true) if flush forwarder.close end |
#count(stat, count, opts = EMPTY_OPTIONS) ⇒ Object
Sends an arbitrary count for the given stat to the statsd server.
200 201 202 203 |
# File 'lib/datadog/statsd.rb', line 200 def count(stat, count, opts = EMPTY_OPTIONS) opts = { sample_rate: opts } if opts.is_a?(Numeric) send_stats(stat, count, COUNTER_TYPE, opts) end |
#decrement(stat, opts = EMPTY_OPTIONS) ⇒ Object
Sends a decrement (count = -1) for the given stat to the statsd server.
187 188 189 190 191 |
# File 'lib/datadog/statsd.rb', line 187 def decrement(stat, opts = EMPTY_OPTIONS) opts = { sample_rate: opts } if opts.is_a?(Numeric) decr_value = - opts.fetch(:by, 1) count(stat, decr_value, opts) end |
#distribution(stat, value, opts = EMPTY_OPTIONS) ⇒ Object
Sends a value to be tracked as a distribution to the statsd server.
245 246 247 |
# File 'lib/datadog/statsd.rb', line 245 def distribution(stat, value, opts = EMPTY_OPTIONS) send_stats(stat, value, DISTRIBUTION_TYPE, opts) end |
#event(title, text, opts = EMPTY_OPTIONS) ⇒ Object
This end point allows you to post events to the stream. You can tag them, set priority and even aggregate them with other events.
Aggregation in the stream is made on hostname/event_type/source_type/aggregation_key. If there’s no event type, for example, then that won’t matter; it will be grouped with other events that don’t have an event type.
335 336 337 338 339 |
# File 'lib/datadog/statsd.rb', line 335 def event(title, text, opts = EMPTY_OPTIONS) telemetry.sent(events: 1) if telemetry forwarder.(serializer.to_event(title, text, opts)) end |
#flush(flush_telemetry: false, sync: false) ⇒ Object
Flush the buffer into the connection
374 375 376 |
# File 'lib/datadog/statsd.rb', line 374 def flush(flush_telemetry: false, sync: false) forwarder.flush(flush_telemetry: flush_telemetry, sync: sync) end |
#gauge(stat, value, opts = EMPTY_OPTIONS) ⇒ Object
Sends an arbitary gauge value for the given stat to the statsd server.
This is useful for recording things like available disk space, memory usage, and the like, which have different semantics than counters.
218 219 220 221 |
# File 'lib/datadog/statsd.rb', line 218 def gauge(stat, value, opts = EMPTY_OPTIONS) opts = { sample_rate: opts } if opts.is_a?(Numeric) send_stats(stat, value, GAUGE_TYPE, opts) end |
#histogram(stat, value, opts = EMPTY_OPTIONS) ⇒ Object
Sends a value to be tracked as a histogram to the statsd server.
232 233 234 |
# File 'lib/datadog/statsd.rb', line 232 def histogram(stat, value, opts = EMPTY_OPTIONS) send_stats(stat, value, HISTOGRAM_TYPE, opts) end |
#host ⇒ Object
382 383 384 |
# File 'lib/datadog/statsd.rb', line 382 def host forwarder.host end |
#increment(stat, opts = EMPTY_OPTIONS) ⇒ Object
Sends an increment (count = 1) for the given stat to the statsd server.
173 174 175 176 177 |
# File 'lib/datadog/statsd.rb', line 173 def increment(stat, opts = EMPTY_OPTIONS) opts = { sample_rate: opts } if opts.is_a?(Numeric) incr_value = opts.fetch(:by, 1) count(stat, incr_value, opts) end |
#port ⇒ Object
386 387 388 |
# File 'lib/datadog/statsd.rb', line 386 def port forwarder.port end |
#service_check(name, status, opts = EMPTY_OPTIONS) ⇒ Object
310 311 312 313 314 |
# File 'lib/datadog/statsd.rb', line 310 def service_check(name, status, opts = EMPTY_OPTIONS) telemetry.sent(service_checks: 1) if telemetry forwarder.(serializer.to_service_check(name, status, opts)) end |
#set(stat, value, opts = EMPTY_OPTIONS) ⇒ Object
Sends a value to be tracked as a set to the statsd server.
294 295 296 297 |
# File 'lib/datadog/statsd.rb', line 294 def set(stat, value, opts = EMPTY_OPTIONS) opts = { sample_rate: opts } if opts.is_a?(Numeric) send_stats(stat, value, SET_TYPE, opts) end |
#socket_path ⇒ Object
390 391 392 |
# File 'lib/datadog/statsd.rb', line 390 def socket_path forwarder.socket_path end |
#sync_with_outbound_io ⇒ Object
369 370 371 |
# File 'lib/datadog/statsd.rb', line 369 def sync_with_outbound_io forwarder.sync_with_outbound_io end |
#tags ⇒ Object
Global tags to be added to every statsd call. Defaults to no tags.
68 69 70 |
# File 'lib/datadog/statsd.rb', line 68 def serializer. end |
#telemetry ⇒ Object
378 379 380 |
# File 'lib/datadog/statsd.rb', line 378 def telemetry forwarder.telemetry end |
#time(stat, opts = EMPTY_OPTIONS) { ... } ⇒ Object
Reports execution time of the provided block using #timing.
If the block fails, the stat is still reported, then the error is reraised
277 278 279 280 281 282 283 |
# File 'lib/datadog/statsd.rb', line 277 def time(stat, opts = EMPTY_OPTIONS) opts = { sample_rate: opts } if opts.is_a?(Numeric) start = now yield ensure timing(stat, ((now - start) * 1000).round, opts) end |
#timing(stat, ms, opts = EMPTY_OPTIONS) ⇒ Object
Sends a timing (in ms) for the given stat to the statsd server. The sample_rate determines what percentage of the time this report is sent. The statsd server then uses the sample_rate to correctly track the average timing for the stat.
259 260 261 262 |
# File 'lib/datadog/statsd.rb', line 259 def timing(stat, ms, opts = EMPTY_OPTIONS) opts = { sample_rate: opts } if opts.is_a?(Numeric) send_stats(stat, ms, TIMING_TYPE, opts) end |
#transport_type ⇒ Object
394 395 396 |
# File 'lib/datadog/statsd.rb', line 394 def transport_type forwarder.transport_type end |